// $Id: dictionarysearchOverlay.js,v 1.16 2005/11/06 17:57:38 jaap Exp $

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Jaap Haitsma <jaap@haitsma.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Uncomment DEBUG part and //log statements to have logging to the console
// Furthermore you need to add the following two lines to prefs.js to see
// the logging in the shell where you started the browser
// user_pref("browser.dom.window.dump.enabled", true);
// user_pref("javascript.options.showInConsole", true);

//////////////////////////////////////
// Debug part
//////////////////////////////////////

/*
function //log(str){
    dump(str + "\n");
}

function dumpObject(obj){
    dump("\n dumping object contents\n");
    for(i in obj){
        dump(i + " = " + obj[i] + "\n");
    }
    dump("\n");
}
*/
//////////////////////////////////////


const NUM_DICTIONARIES=4;


var dictionarySearchStrBundle;

// This holds the object for accessing the preferences stored in prefs.js
var dictionarySearchPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService).getBranch("dictionarysearch.");

var gInThunderbird = false;


// Every time a new browser window is made dictionarySearchInit will be called
window.addEventListener("load",dictionarySearchInit,false);


function dictionarySearchInit() {
    //log("dictionarySearchInit()");
     //log (navigator.userAgent);
    if (navigator.userAgent.search(/Thunderbird/gi) != -1){
        gInThunderbird = true;
    } 

    //log("gInThunderbird " + gInThunderbird);

    dictionarySearchStrBundle = document.getElementById("dictionarysearch_string_bundle");

    if (!dictionarySearchPrefs.prefHasUserValue("menutext1")){
        ////log("dictionarySearchInit(): no preferences found in prefs.js taking defaults");
        // Assign default values
        dictionarySearchPrefs.setCharPref("url1", dictionarySearchStrBundle.getString("dictionarysearch.default.URL"));
        dictionarySearchPrefs.setCharPref("menutext1", dictionarySearchStrBundle.getString("dictionarysearch.default.menutext"));
    }
    for (i=2; i<=NUM_DICTIONARIES; i++){
        if (!dictionarySearchPrefs.prefHasUserValue("menutext"+i)){
            dictionarySearchPrefs.setCharPref("url"+i, "");
            dictionarySearchPrefs.setCharPref("menutext"+i, "");
        }
    }            
     
    
    if (document.getElementById("contentAreaContextMenu")){
        document.getElementById("contentAreaContextMenu").addEventListener("popupshowing",onDictionarySearchPopup,false);
    }
    else if (document.getElementById("messagePaneContext")){
        document.getElementById("messagePaneContext").addEventListener("popupshowing",onDictionarySearchPopup,false);    
    }
    else if (document.getElementById("msgComposeContext")){
        document.getElementById("msgComposeContext").addEventListener("popupshowing",onDictionarySearchPopup,false);    
    }
            
}


function onDictionarySearchPopup() { 
    //log("onDictionarySearchPopup()");
    // hide the Dictionary Search item when apropriate (use same logic as for web search)
    var selectedText = dictionarySearchGetSelectedText(" ");
       
    if (selectedText!=""){
        //log ("onDictionarySearchPopup() text selected");
        if (selectedText.length > 18){
            selectedText = selectedText.substr(0,14) + "...";
        }
        var menuText;
        var item;

        for (i=1 ;i<=NUM_DICTIONARIES; i++){
            menuText = dictionarySearchPrefs.getCharPref("menutext"+i);
            item = document.getElementById("context_dictionarysearchselect"+i);
            item.hidden = true;
            if (menuText!=""){        
                item.hidden = false;        
                menuText = menuText.split("$");
                menuText = menuText[0] + selectedText + menuText[1];
                item.setAttribute("label", menuText);
            }
        }
    }
    else{

       for (i=1 ;i<=NUM_DICTIONARIES; i++){
            item = document.getElementById("context_dictionarysearchselect"+i);
            item.hidden = true;
       }
    }

}

function onDictionarySearch(n){ 
    var url = dictionarySearchPrefs.getCharPref("url"+n);
    url = url.split("$");
    url = url[0] + dictionarySearchGetSelectedText("+").toLocaleLowerCase() + url[1];
    //log("onDictionarySearch() : url = " + url);

    if (!gInThunderbird){
        openNewTabWith(url, null, null, true);
    }
    else{
        var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();         
        messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
        messenger.launchExternalURL(url);         
    }
}


function dictionarySearchGetSelectedText(concationationChar){
    //log("dictionarySearchGetSelectedText()");
    
    var node = document.popupNode;
    var selection = "";

    if ((node instanceof HTMLTextAreaElement) || (node instanceof HTMLInputElement && node.type == "text")) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } 
    else {
        var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow, 'document', 'getSelection()');
        selection = focusedWindow.getSelection().toString();
    }

    // Limit length to 150 to optimize performance. Longer does not make sense
    if (selection.length>=150){
        selection = selection.substring(0, 149);
    }
    selection = selection.replace(/(\n|\r|\t)+/g, " ");
    // Strip spaces at start and end.
    selection = selection.replace(/(^\s+)|(\s+$)/g, "");

    selection = selection.split(" ");
   
    // Remove certain characters at the beginning and end of every word
    for (i=0; i<selection.length; i++){
        selection[i]=selection[i].replace(/^(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+/, "");
        selection[i]=selection[i].replace(/(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+$/, "");
    }   
    selection = selection.join(concationationChar);   
    return selection; 
}

function dictionarySearchSaveSettings(){
    //log("dictionarySearchSaveSettings()");

    var url;
    var menuText;
    var accessKey;
    for (i=1; i<=NUM_DICTIONARIES; i++){
        url = document.getElementById("dictionarysearchURL"+i).value;
        menuText = document.getElementById("dictionarysearchMenuText"+i).value;
        if (menuText != ""){       
            if (!dictionarySearchCheckDollarSign(menuText)){
                alert(dictionarySearchStrBundle.getString("dictionarysearch.menutext.error1")  + " " + i
                      + " " + dictionarySearchStrBundle.getString("dictionarysearch.menutext.error2"));
                return false;
            }

            if (!dictionarySearchCheckDollarSign(url)){
                alert(dictionarySearchStrBundle.getString("dictionarysearch.URL.error1") + " " + i
                  + " " + dictionarySearchStrBundle.getString("dictionarysearch.URL.error2"));
                return false;
            }

        }
        dictionarySearchPrefs.setCharPref("url"+i, url);
        dictionarySearchPrefs.setCharPref("menutext"+i, menuText);
    }
    return true;
}

function dictionarySearchLoadSettings(){
    //log("dictionarySearchLoadSettings()");
    for (i=1; i<=NUM_DICTIONARIES; i++){
        document.getElementById("dictionarysearchURL"+i).value = dictionarySearchPrefs.getCharPref("url"+i);
        document.getElementById("dictionarysearchMenuText"+i).value = dictionarySearchPrefs.getCharPref("menutext"+i);
    }
}


function dictionarySearchCheckDollarSign(str){
    //log("dictionarySearchCheckDollarSign(str)");

    ind = str.indexOf('$');
    if (ind == -1){
        return false;
    }
    if (ind != str.lastIndexOf('$')){
        return false;
    }
    return true;
}
